@@ -19,6 +19,8 @@ pub fn typed_string_ref_params(
     let Parameters {
         owned_type,
         check_mode,
+        omit_debug,
+        omit_display,
         derive_serde,
     } = params;
 
@@ -29,18 +31,20 @@ pub fn typed_string_ref_params(
         .as_ref()
         .map(|owned_type| comparison_impls(&body.ident, owned_type));
     let conversion_impls = conversion_impls(&body.ident, &check_mode);
-    let common_impls = common_impls(&body.ident);
+    let display_impl = (!omit_display).then(|| display_impl(&body.ident));
+    let debug_impl = (!omit_debug).then(|| debug_impl(&body.ident));
     let serde_impls = derive_serde.then(|| serde_impls(&body.ident, &owned_type, &check_mode));
 
     let output = quote! {
-        #[derive(Debug, Hash, PartialEq, Eq)]
+        #[derive(Hash, PartialEq, Eq)]
         #[repr(transparent)]
         #body
 
         #inherent_impl
         #comparison_impls
         #conversion_impls
-        #common_impls
+        #debug_impl
+        #display_impl
         #serde_impls
     };
 
@@ -411,7 +415,7 @@ fn comparison_impls(name: &syn::Ident, owned_type: &syn::Type) -> proc_macro2::T
 }
 
 fn conversion_impls(name: &syn::Ident, check_mode: &CheckMode) -> proc_macro2::TokenStream {
-    match check_mode {
+    let from_str = match check_mode {
         CheckMode::None => quote! {
             impl<'a> From<&'a str> for &'a #name {
                 fn from(s: &'a str) -> &'a #name {
@@ -443,22 +447,36 @@ fn conversion_impls(name: &syn::Ident, check_mode: &CheckMode) -> proc_macro2::T
                 }
             }
         }
+    };
+
+    quote! {
+        #from_str
+
+        impl<'a> From<&'a #name> for ::std::borrow::Cow<'a, #name> {
+            fn from(r: &'a #name) -> Self {
+                ::std::borrow::Cow::Borrowed(r)
+            }
+        }
     }
 }
 
-fn common_impls(name: &syn::Ident) -> proc_macro2::TokenStream {
+fn display_impl(name: &syn::Ident) -> proc_macro2::TokenStream {
     quote! {
-        impl AsRef<str> for #name {
-            #[inline]
-            fn as_ref(&self) -> &str {
-                self.as_str()
-            }
-        }
-
         impl ::std::fmt::Display for #name {
             #[inline]
             fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
-                f.write_str(&self.0)
+                <str as ::std::fmt::Display>::fmt(&self.0, f)
+            }
+        }
+    }
+}
+
+fn debug_impl(name: &syn::Ident) -> proc_macro2::TokenStream {
+    quote! {
+        impl ::std::fmt::Debug for #name {
+            #[inline]
+            fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
+                <str as ::std::fmt::Debug>::fmt(&self.0, f)
             }
         }
     }
@@ -541,6 +559,7 @@ pub fn serde_impls(
 #[cfg(test)]
 mod tests {
     use super::*;
+    use pretty_assertions::assert_eq;
     use quote::format_ident;
     use syn::parse_quote;
 
@@ -569,6 +588,7 @@ mod tests {
                 }
             }
 
+            #[allow(clippy::needless_question_mark)]
             impl<'de: 'a, 'a> ::serde::Deserialize<'de> for &'a Borrowed {
                 fn deserialize<D: ::serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
                     let raw = <&str as ::serde::Deserialize<'de>>::deserialize(deserializer)?;
@@ -600,6 +620,7 @@ mod tests {
                 }
             }
 
+            #[allow(clippy::needless_question_mark)]
             impl<'de: 'a, 'a> ::serde::Deserialize<'de> for &'a Borrowed {
                 fn deserialize<D: ::serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
                     let raw = <&str as ::serde::Deserialize<'de>>::deserialize(deserializer)?;
@@ -630,6 +651,7 @@ mod tests {
                 }
             }
 
+            #[allow(clippy::needless_question_mark)]
             impl<'de: 'a, 'a> ::serde::Deserialize<'de> for &'a Borrowed {
                 fn deserialize<D: ::serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
                     let raw = <&str as ::serde::Deserialize<'de>>::deserialize(deserializer)?;
@@ -653,6 +675,7 @@ mod tests {
                 }
             }
 
+            #[allow(clippy::needless_question_mark)]
             impl<'de: 'a, 'a> ::serde::Deserialize<'de> for &'a Borrowed {
                 fn deserialize<D: ::serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
                     let raw = <&str as ::serde::Deserialize<'de>>::deserialize(deserializer)?;
