@@ -18,6 +18,9 @@ pub fn typed_string_params(
 ) -> Result<proc_macro2::TokenStream, syn::Error> {
     let Parameters {
         ref_type,
+        omit_clone,
+        impl_debug,
+        impl_display,
         derive_serde,
         no_auto_ref,
         ref_doc,
@@ -41,20 +44,33 @@ pub fn typed_string_params(
                 name,
                 &body.vis,
                 &ref_type,
-                check_mode,
-                derive_serde,
+                crate::borrow::Parameters {
+                    owned_type: Some(syn::parse_quote!(#name)),
+                    check_mode,
+                    omit_debug: impl_debug != parameters::AutoImplOption::Auto,
+                    omit_display: impl_display != parameters::AutoImplOption::Auto,
+                    derive_serde,
+                },
                 ref_doc,
             )
         })
         .transpose()?;
 
+    let clone = (!omit_clone).then(|| quote! { Clone, });
+    let debug_impl =
+        (impl_debug != parameters::AutoImplOption::None).then(|| debug_impl(name, &ref_type));
+    let display_impl =
+        (impl_debug != parameters::AutoImplOption::None).then(|| display_impl(name, &ref_type));
+
     let output = quote! {
-        #[derive(Clone, Debug, Hash, PartialEq, Eq)]
+        #[derive(#clone Hash, PartialEq, Eq)]
         #[repr(transparent)]
         #body
 
         #inherent_impls
         #common_impls
+        #debug_impl
+        #display_impl
         #conversion_impls
         #serde_impls
 
@@ -240,8 +256,7 @@ fn construct_ref_item(
     name: &syn::Ident,
     vis: &syn::Visibility,
     ref_type: &syn::Type,
-    check_mode: CheckMode,
-    derive_serde: bool,
+    params: crate::borrow::Parameters,
     ref_doc: Option<String>,
 ) -> Result<proc_macro2::TokenStream, syn::Error> {
     let ref_vis = vis.clone();
@@ -249,11 +264,7 @@ fn construct_ref_item(
     let ref_doc = ref_doc.unwrap_or_else(|| format!("A reference to a borrowed [`{}`]", name));
 
     crate::borrow::typed_string_ref_params(
-        crate::borrow::Parameters {
-            owned_type: Some(syn::parse_quote!(#name)),
-            check_mode,
-            derive_serde,
-        },
+        params,
         syn::parse_quote! {
                 #[doc = #ref_doc]
                 #ref_vis struct #ref_type(str);
@@ -261,6 +272,28 @@ fn construct_ref_item(
     )
 }
 
+pub fn display_impl(name: &syn::Ident, ref_type: &syn::Type) -> proc_macro2::TokenStream {
+    quote! {
+        impl<'a> ::std::fmt::Display for #name {
+            #[inline]
+            fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
+                <#ref_type as ::std::fmt::Display>::fmt(::std::borrow::Borrow::borrow(self), f)
+            }
+        }
+    }
+}
+
+pub fn debug_impl(name: &syn::Ident, ref_type: &syn::Type) -> proc_macro2::TokenStream {
+    quote! {
+        impl<'a> ::std::fmt::Debug for #name {
+            #[inline]
+            fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
+                <#ref_type as ::std::fmt::Debug>::fmt(::std::borrow::Borrow::borrow(self), f)
+            }
+        }
+    }
+}
+
 pub fn common_impls(name: &syn::Ident, ref_type: &syn::Type) -> proc_macro2::TokenStream {
     quote! {
         impl From<&'_ #ref_type> for #name {
@@ -277,26 +310,12 @@ pub fn common_impls(name: &syn::Ident, ref_type: &syn::Type) -> proc_macro2::Tok
             }
         }
 
-        impl AsRef<str> for #name {
-            #[inline]
-            fn as_ref(&self) -> &str {
-                self.as_str()
-            }
-        }
-
         impl AsRef<#ref_type> for #name {
             #[inline]
             fn as_ref(&self) -> &#ref_type {
                 ::std::ops::Deref::deref(self)
             }
         }
-
-        impl<'a> ::std::fmt::Display for #name {
-            #[inline]
-            fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
-                f.write_str(self.as_str())
-            }
-        }
     }
 }
 
@@ -446,7 +465,7 @@ fn conversion_impls(
     wrapped_type: &syn::Type,
     check_mode: &CheckMode,
 ) -> proc_macro2::TokenStream {
-    match check_mode {
+    let impls = match check_mode {
         CheckMode::None => infallible_conversion_impls(name, ref_type, wrapped_type),
         CheckMode::Validate(validator) => {
             fallible_conversion_impls(name, ref_type, wrapped_type, validator)
@@ -454,6 +473,31 @@ fn conversion_impls(
         CheckMode::Normalize(normalizer) => {
             normalized_conversion_impls(name, ref_type, wrapped_type, normalizer)
         }
+    };
+
+    quote! {
+        #impls
+
+        impl From<#name> for Box<#ref_type> {
+            fn from(r: #name) -> Self {
+                r.into_boxed_ref()
+            }
+        }
+
+        impl From<Box<#ref_type>> for #name {
+            fn from(r: Box<#ref_type>) -> Self {
+                r.into_owned()
+            }
+        }
+
+        impl<'a> From<::std::borrow::Cow<'a, #ref_type>> for #name {
+            fn from(r: ::std::borrow::Cow<'a, #ref_type>) -> Self {
+                match r {
+                    ::std::borrow::Cow::Borrowed(b) => b.to_owned(),
+                    ::std::borrow::Cow::Owned(o) => o,
+                }
+            }
+        }
     }
 }
 
@@ -488,6 +532,7 @@ pub fn serde_impls(
 #[cfg(test)]
 mod tests {
     use super::*;
+    use pretty_assertions::assert_eq;
     use quote::format_ident;
     use syn::parse_quote;
 
@@ -520,6 +565,7 @@ mod tests {
                 }
             }
 
+            #[allow(clippy::needless_question_mark)]
             impl<'de> ::serde::Deserialize<'de> for Owned {
                 fn deserialize<D: ::serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
                     let raw = <Wrapped as ::serde::Deserialize<'de>>::deserialize(deserializer)?;
@@ -544,6 +590,7 @@ mod tests {
                 }
             }
 
+            #[allow(clippy::needless_question_mark)]
             impl<'de> ::serde::Deserialize<'de> for Owned {
                 fn deserialize<D: ::serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
                     let raw = <Wrapped as ::serde::Deserialize<'de>>::deserialize(deserializer)?;
